@@ -36,6 +36,7 @@ from weconnect.elements.range_status import RangeStatus
 from weconnect.elements.window_heating_status import WindowHeatingStatus
 from weconnect.elements.odometer_measurement import OdometerMeasurement
 from weconnect.elements.range_measurements import RangeMeasurements
+from weconnect.elements.readiness_status import ReadinessStatus
 from weconnect.errors import APICompatibilityError, RetrievalError, APIError
 from weconnect.util import toBool
 from weconnect.weconnect_errors import ErrorEventType
@@ -62,7 +63,13 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         self.enrollmentStatus: AddressableAttribute[Vehicle.User.EnrollmentStatus] = AddressableAttribute(localAddress='enrollmentStatus', parent=self,
                                                                                                           value=None,
                                                                                                           valueType=Vehicle.User.EnrollmentStatus)
+        self.userRoleStatus: AddressableAttribute[Vehicle.User.UserRoleStatus] = AddressableAttribute(localAddress='userRoleStatus', parent=self,
+                                                                                                      value=None,
+                                                                                                      valueType=Vehicle.User.UserRoleStatus)
         self.model: AddressableAttribute[str] = AddressableAttribute(localAddress='model', parent=self, value=None, valueType=str)
+        self.devicePlatform: AddressableAttribute[Vehicle.DevicePlatform] = AddressableAttribute(localAddress='devicePlatform', parent=self,
+                                                                                                 value=None,
+                                                                                                 valueType=Vehicle.DevicePlatform)
         self.nickname: AddressableAttribute[str] = AddressableAttribute(localAddress='nickname', parent=self, value=None, valueType=str)
         self.capabilities: AddressableDict[str, GenericCapability] = AddressableDict(localAddress='capabilities', parent=self)
         self.statuses: AddressableDict[str, GenericStatus] = AddressableDict(localAddress='status', parent=self)
@@ -113,11 +120,29 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             else:
                 self.enrollmentStatus.enabled = False
 
+            if 'userRoleStatus' in fromDict and fromDict['userRoleStatus']:
+                try:
+                    self.userRoleStatus.setValueWithCarTime(Vehicle.User.UserRoleStatus(fromDict['userRoleStatus']), lastUpdateFromCar=None, fromServer=True)
+                except ValueError:
+                    self.userRoleStatus.setValueWithCarTime(Vehicle.User.UserRoleStatus.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
+                    LOG.warning('An unsupported userRoleStatus: %s was provided, please report this as a bug', fromDict['userRoleStatus'])
+            else:
+                self.userRoleStatus.enabled = False
+
             if 'model' in fromDict:
                 self.model.setValueWithCarTime(fromDict['model'], lastUpdateFromCar=None, fromServer=True)
             else:
                 self.model.enabled = False
 
+            if 'devicePlatform' in fromDict and fromDict['devicePlatform']:
+                try:
+                    self.devicePlatform.setValueWithCarTime(Vehicle.DevicePlatform(fromDict['devicePlatform']), lastUpdateFromCar=None, fromServer=True)
+                except ValueError:
+                    self.devicePlatform.setValueWithCarTime(Vehicle.DevicePlatform.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
+                    LOG.warning('An unsupported devicePlatform: %s was provided, please report this as a bug', fromDict['devicePlatform'])
+            else:
+                self.devicePlatform.enabled = False
+
             if 'nickname' in fromDict:
                 self.nickname.setValueWithCarTime(fromDict['nickname'], lastUpdateFromCar=None, fromServer=True)
             else:
@@ -166,7 +191,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                if key not in ['vin',
                                               'role',
                                               'enrollmentStatus',
+                                              'userRoleStatus',
                                               'model',
+                                              'devicePlatform',
                                               'nickname',
                                               'capabilities',
                                               'images',
@@ -199,6 +226,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             except exceptions.ConnectionError as connectionError:
                 self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection', 'Could not fetch vehicle status due to connection problem')
                 raise RetrievalError from connectionError
+            except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                           'Could not refresh token due to connection problem with chunked encoding')
+                raise RetrievalError from chunkedEncodingError
             except exceptions.ReadTimeout as timeoutError:
                 self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle status due to timeout')
                 raise RetrievalError from timeoutError
@@ -218,6 +249,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                                'Could not fetch vehicle status due to connection problem')
                     raise RetrievalError from connectionError
+                except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                    self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                               'Could not refresh token due to connection problem with chunked encoding')
+                    raise RetrievalError from chunkedEncodingError
                 except exceptions.ReadTimeout as timeoutError:
                     self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle status due to timeout')
                     raise RetrievalError from timeoutError
@@ -253,6 +288,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                                        'maintenanceStatus': MaintenanceStatus,
                                                        'rangeStatus': RangeStatus,
                                                        'capabilityStatus': CapabilityStatus,
+                                                       'capabilitiesStatus': GenericStatus,
                                                        'climatisationTimer': ClimatizationTimer,
                                                        'climatisationRequestStatus': GenericRequestStatus,
                                                        'chargingSettingsRequestStatus': GenericRequestStatus,
@@ -262,7 +298,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                                        'rangeMeasurements': RangeMeasurements,
                                                        'odometerMeasurement': OdometerMeasurement,
                                                        'oilLevelStatus': GenericStatus,
-                                                       'measurements': GenericStatus
+                                                       'measurements': GenericStatus,
+                                                       'readinessBatterySupportStatus': GenericStatus,
+                                                       'readinessStatus': ReadinessStatus
                                                        }
         if data is not None and 'data' in data and data['data']:
             for key, className in keyClassMap.items():
@@ -324,33 +362,15 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         # Controls
         self.controls.update()
 
-        data = None
-        cacheDate = None
-        url = 'https://mobileapi.apps.emea.vwapps.io/vehicles/' + self.vin.value + '/parkingposition'
-        if not force and (self.weConnect.maxAge is not None and self.weConnect.cache is not None and url in self.weConnect.cache):
-            data, cacheDateString = self.weConnect.cache[url]
-            cacheDate = datetime.fromisoformat(cacheDateString)
-        if data is None or self.weConnect.maxAge is None \
-                or (cacheDate is not None and cacheDate < (datetime.utcnow() - timedelta(seconds=self.weConnect.maxAge))):
-            try:
-                statusResponse = self.weConnect.session.get(url, allow_redirects=False)
-                self.weConnect.recordElapsed(statusResponse.elapsed)
-            except exceptions.ConnectionError as connectionError:
-                self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
-                                           'Could not fetch parking position due to connection problem')
-                raise RetrievalError from connectionError
-            except exceptions.ReadTimeout as timeoutError:
-                self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch parking position due to timeout')
-                raise RetrievalError from timeoutError
-            except exceptions.RetryError as retryError:
-                raise RetrievalError from retryError
-            if statusResponse.status_code == codes['ok']:
-                data = statusResponse.json()
-                if self.weConnect.cache is not None:
-                    self.weConnect.cache[url] = (data, str(datetime.utcnow()))
-            elif statusResponse.status_code == codes['unauthorized']:
-                LOG.info('Server asks for new authorization')
-                self.weConnect.login()
+        if not updateCapabilities or ('parkingPosition' in self.capabilities and self.capabilities['parkingPosition'].status.value is None):
+            data = None
+            cacheDate = None
+            url = 'https://mobileapi.apps.emea.vwapps.io/vehicles/' + self.vin.value + '/parkingposition'
+            if not force and (self.weConnect.maxAge is not None and self.weConnect.cache is not None and url in self.weConnect.cache):
+                data, cacheDateString = self.weConnect.cache[url]
+                cacheDate = datetime.fromisoformat(cacheDateString)
+            if data is None or self.weConnect.maxAge is None \
+                    or (cacheDate is not None and cacheDate < (datetime.utcnow() - timedelta(seconds=self.weConnect.maxAge))):
                 try:
                     statusResponse = self.weConnect.session.get(url, allow_redirects=False)
                     self.weConnect.recordElapsed(statusResponse.elapsed)
@@ -358,6 +378,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                                'Could not fetch parking position due to connection problem')
                     raise RetrievalError from connectionError
+                except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                    self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                               'Could not refresh token due to connection problem with chunked encoding')
+                    raise RetrievalError from chunkedEncodingError
                 except exceptions.ReadTimeout as timeoutError:
                     self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch parking position due to timeout')
                     raise RetrievalError from timeoutError
@@ -367,40 +391,63 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     data = statusResponse.json()
                     if self.weConnect.cache is not None:
                         self.weConnect.cache[url] = (data, str(datetime.utcnow()))
+                elif statusResponse.status_code == codes['unauthorized']:
+                    LOG.info('Server asks for new authorization')
+                    self.weConnect.login()
+                    try:
+                        statusResponse = self.weConnect.session.get(url, allow_redirects=False)
+                        self.weConnect.recordElapsed(statusResponse.elapsed)
+                    except exceptions.ConnectionError as connectionError:
+                        self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
+                                                   'Could not fetch parking position due to connection problem')
+                        raise RetrievalError from connectionError
+                    except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                        self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                                   'Could not refresh token due to connection problem with chunked encoding')
+                        raise RetrievalError from chunkedEncodingError
+                    except exceptions.ReadTimeout as timeoutError:
+                        self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch parking position due to timeout')
+                        raise RetrievalError from timeoutError
+                    except exceptions.RetryError as retryError:
+                        raise RetrievalError from retryError
+                    if statusResponse.status_code == codes['ok']:
+                        data = statusResponse.json()
+                        if self.weConnect.cache is not None:
+                            self.weConnect.cache[url] = (data, str(datetime.utcnow()))
+                    else:
+                        self.weConnect.notifyError(self, ErrorEventType.HTTP, str(statusResponse.status_code),
+                                                   'Could not fetch parking position due to server error')
+                        raise RetrievalError('Could not retrieve parking position even after re-authorization.'
+                                             f' Status Code was: {statusResponse.status_code}')
+                elif statusResponse.status_code == codes['bad_request'] \
+                        or statusResponse.status_code == codes['no_content'] \
+                        or statusResponse.status_code == codes['not_found'] \
+                        or statusResponse.status_code == codes['bad_gateway'] \
+                        or statusResponse.status_code == codes['forbidden']:
+                    try:
+                        data = statusResponse.json()
+                    except json.JSONDecodeError:
+                        data = None
                 else:
                     self.weConnect.notifyError(self, ErrorEventType.HTTP, str(statusResponse.status_code),
                                                'Could not fetch parking position due to server error')
-                    raise RetrievalError('Could not retrieve parking position even after re-authorization.'
-                                         f' Status Code was: {statusResponse.status_code}')
-            elif statusResponse.status_code == codes['bad_request'] \
-                    or statusResponse.status_code == codes['no_content'] \
-                    or statusResponse.status_code == codes['not_found'] \
-                    or statusResponse.status_code == codes['bad_gateway'] \
-                    or statusResponse.status_code == codes['forbidden']:
-                try:
-                    data = statusResponse.json()
-                except json.JSONDecodeError:
-                    data = None
-            else:
-                self.weConnect.notifyError(self, ErrorEventType.HTTP, str(statusResponse.status_code),
-                                           'Could not fetch parking position due to server error')
-                raise RetrievalError(f'Could not retrieve parking position. Status Code was: {statusResponse.status_code}')
+                    raise RetrievalError(f'Could not retrieve parking position. Status Code was: {statusResponse.status_code}')
 
-        if data is not None and 'data' in data:
-            if 'parkingPosition' in self.statuses:
-                self.statuses['parkingPosition'].update(fromDict=data['data'])
-            else:
-                self.statuses['parkingPosition'] = ParkingPosition(vehicle=self,
-                                                                   parent=self.statuses,
-                                                                   statusId='parkingPosition',
-                                                                   fromDict=data['data'])
-        elif 'parkingPosition' in self.statuses:
-            parkingPosition: ParkingPosition = cast(ParkingPosition, self.statuses['parkingPosition'])
-            parkingPosition.latitude.enabled = False
-            parkingPosition.longitude.enabled = False
-            parkingPosition.carCapturedTimestamp.setValueWithCarTime(None, fromServer=True)
-            parkingPosition.carCapturedTimestamp.enabled = False
-            parkingPosition.enabled = False
+            if data is not None and 'data' in data:
+                if 'parkingPosition' in self.statuses:
+                    self.statuses['parkingPosition'].update(fromDict=data['data'])
+                else:
+                    self.statuses['parkingPosition'] = ParkingPosition(vehicle=self,
+                                                                       parent=self.statuses,
+                                                                       statusId='parkingPosition',
+                                                                       fromDict=data['data'])
+            elif 'parkingPosition' in self.statuses:
+                parkingPosition: ParkingPosition = cast(ParkingPosition, self.statuses['parkingPosition'])
+                parkingPosition.latitude.enabled = False
+                parkingPosition.longitude.enabled = False
+                parkingPosition.carCapturedTimestamp.setValueWithCarTime(None, fromServer=True)
+                parkingPosition.carCapturedTimestamp.enabled = False
+                parkingPosition.enabled = False
 
         # error handling
         if data is not None and 'error' in data and data['error']:
@@ -435,6 +482,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                            'Could not fetch vehicle image list due to connection problem')
                 raise RetrievalError from connectionError
+            except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                           'Could not refresh token due to connection problem with chunked encoding')
+                raise RetrievalError from chunkedEncodingError
             except exceptions.ReadTimeout as timeoutError:
                 self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle image list due to timeout')
                 raise RetrievalError from timeoutError
@@ -454,6 +505,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                                'Could not fetch vehicle image list due to connection problem')
                     raise RetrievalError from connectionError
+                except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                    self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                               'Could not refresh token due to connection problem with chunked encoding')
+                    raise RetrievalError from chunkedEncodingError
                 except exceptions.ReadTimeout as timeoutError:
                     self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle image list due to timeout')
                     raise RetrievalError from timeoutError
@@ -490,6 +545,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                         self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                                    'Could not fetch vehicle image due to connection problem')
                         raise RetrievalError from connectionError
+                    except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                        self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                                   'Could not refresh token due to connection problem with chunked encoding')
+                        raise RetrievalError from chunkedEncodingError
                     except exceptions.ReadTimeout as timeoutError:
                         self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle image due to timeout')
                         raise RetrievalError from timeoutError
@@ -512,6 +571,10 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                             self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'connection',
                                                        'Could not fetch vehicle image due to connection problem')
                             raise RetrievalError from connectionError
+                        except exceptions.ChunkedEncodingError as chunkedEncodingError:
+                            self.weConnect.notifyError(self, ErrorEventType.CONNECTION, 'chunked encoding error',
+                                                       'Could not refresh token due to connection problem with chunked encoding')
+                            raise RetrievalError from chunkedEncodingError
                         except exceptions.ReadTimeout as timeoutError:
                             self.weConnect.notifyError(self, ErrorEventType.TIMEOUT, 'timeout', 'Could not fetch vehicle image due to timeout')
                             raise RetrievalError from timeoutError
@@ -714,12 +777,16 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             returnString += f'VIN:               {self.vin.value}\n'
         if self.model.enabled and self.model.value is not None:
             returnString += f'Model:             {self.model.value}\n'
+        if self.devicePlatform.enabled and self.devicePlatform.value is not None:
+            returnString += f'Device Platform:   {self.devicePlatform.value.value}\n'
         if self.nickname.enabled and self.nickname.value is not None:
             returnString += f'Nickname:          {self.nickname.value}\n'
         if self.role.enabled and self.role.value is not None:
             returnString += f'Role:              {self.role.value.value}\n'  # pylint: disable=no-member
         if self.enrollmentStatus.enabled and self.enrollmentStatus.value is not None:
             returnString += f'Enrollment Status: {self.enrollmentStatus.value.value}\n'  # pylint: disable=no-member
+        if self.userRoleStatus.enabled and self.userRoleStatus.value is not None:
+            returnString += f'User Role Status:  {self.userRoleStatus.value.value}\n'  # pylint: disable=no-member
         if self.coUsers.enabled:
             returnString += f'Co-Users: {len(self.coUsers)} items\n'
             for coUser in self.coUsers:
@@ -748,6 +815,11 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         VENTILATING = 'ventilating'
         WARNING = 'warning'
 
+    class DevicePlatform(Enum,):
+        MBB_ODP = 'MBB_ODP'
+        WCAR = 'WCAR'
+        UNKNOWN = 'UNKNOWN'
+
     class User(AddressableObject):
         def __init__(
             self,
@@ -817,7 +889,13 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             UNKNOWN = 'UNKNOWN'
 
         class EnrollmentStatus(Enum,):
+            STARTED = 'STARTED'
             NOT_STARTED = 'NOT_STARTED'
             COMPLETED = 'COMPLETED'
             GDC_MISSING = 'GDC_MISSING'
+            INACTIVE = 'INACTIVE'
+            UNKNOWN = 'UNKNOWN'
+
+        class UserRoleStatus(Enum,):
+            ENABLED = 'ENABLED'
             UNKNOWN = 'UNKNOWN'
